/*
 * Copyright (C) 2006  Sony Computer Entertainment Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifndef _PS3AV_H_
#define _PS3AV_H_

/** command for ioctl() **/
#define PS3AV_VERSION 0x205 /* version of ps3av command */


#define PS3AV_CID_AV_INIT              0x00000001
#define PS3AV_CID_AV_FIN               0x00000002
#define PS3AV_CID_AV_GET_HW_CONF       0x00000003
#define PS3AV_CID_AV_GET_MONITOR_INFO  0x00000004
#define PS3AV_CID_AV_ENABLE_EVENT      0x00000006
#define PS3AV_CID_AV_DISABLE_EVENT     0x00000007
#define PS3AV_CID_AV_TV_MUTE           0x0000000a

#define PS3AV_CID_AV_VIDEO_CS          0x00010001
#define PS3AV_CID_AV_VIDEO_MUTE        0x00010002
#define PS3AV_CID_AV_VIDEO_DISABLE_SIG 0x00010003
#define PS3AV_CID_AV_AUDIO_PARAM       0x00020001
#define PS3AV_CID_AV_AUDIO_MUTE        0x00020002
#define PS3AV_CID_AV_HDMI_MODE         0x00040001

#define PS3AV_CID_VIDEO_INIT           0x01000001
#define PS3AV_CID_VIDEO_MODE           0x01000002
#define PS3AV_CID_VIDEO_FORMAT         0x01000004
#define PS3AV_CID_VIDEO_PITCH          0x01000005

#define PS3AV_CID_AUDIO_INIT           0x02000001
#define PS3AV_CID_AUDIO_MODE           0x02000002
#define PS3AV_CID_AUDIO_MUTE           0x02000003
#define PS3AV_CID_AUDIO_ACTIVE         0x02000004
#define PS3AV_CID_AUDIO_INACTIVE       0x02000005
#define PS3AV_CID_AUDIO_SPDIF_BIT      0x02000006
#define PS3AV_CID_AUDIO_CTRL           0x02000007

#define PS3AV_CID_EVENT_UNPLUGGED      0x10000001
#define PS3AV_CID_EVENT_PLUGGED        0x10000002
#define PS3AV_CID_EVENT_HDCP_DONE      0x10000003
#define PS3AV_CID_EVENT_HDCP_FAIL      0x10000004
#define PS3AV_CID_EVENT_HDCP_AUTH      0x10000005
#define PS3AV_CID_EVENT_HDCP_ERROR     0x10000006

#define PS3AV_CID_AVB_PARAM            0x04000001

/* max backend ports */
#define PS3AV_HDMI_MAX                 2 /* HDMI_0 HDMI_1 */
#define PS3AV_AVMULTI_MAX              1 /* AVMULTI_0 */
#define PS3AV_AV_PORT_MAX              (PS3AV_HDMI_MAX + PS3AV_AVMULTI_MAX) 
#define PS3AV_OPT_PORT_MAX             1 /* SPDIF0 */
#define PS3AV_HEAD_MAX                 2 /* HEAD_A HEAD_B */

/* num of pkt for PS3AV_CID_AVB_PARAM */
#define PS3AV_AVB_NUM_VIDEO            PS3AV_HEAD_MAX
#define PS3AV_AVB_NUM_AUDIO            0 /* not supported */
#define PS3AV_AVB_NUM_AV_VIDEO         PS3AV_AV_PORT_MAX 
#define PS3AV_AVB_NUM_AV_AUDIO         PS3AV_HDMI_MAX

#define PS3AV_MUTE_PORT_MAX            1 /* num of ports in mute pkt */

/* event_bit */
#define PS3AV_CMD_EVENT_BIT_UNPLUGGED			(1 << 0)
#define PS3AV_CMD_EVENT_BIT_PLUGGED			(1 << 1)
#define PS3AV_CMD_EVENT_BIT_HDCP_DONE			(1 << 2)
#define PS3AV_CMD_EVENT_BIT_HDCP_FAIL			(1 << 3)
#define PS3AV_CMD_EVENT_BIT_HDCP_REAUTH			(1 << 4)
#define PS3AV_CMD_EVENT_BIT_HDCP_TOPOLOGY		(1 << 5)

/* common params */
/* mute */
#define PS3AV_CMD_MUTE_OFF				0x0000
#define PS3AV_CMD_MUTE_ON				0x0001
/* avport */
#define PS3AV_CMD_AVPORT_HDMI_0				0x0000
#define PS3AV_CMD_AVPORT_HDMI_1				0x0001
#define PS3AV_CMD_AVPORT_AVMULTI_0			0x0010
#define PS3AV_CMD_AVPORT_SPDIF_0			0x0020
#define PS3AV_CMD_AVPORT_SPDIF_1			0x0021

/* for av backend */
/* av_mclk */
#define PS3AV_CMD_AV_MCLK_128				0x0000
#define PS3AV_CMD_AV_MCLK_256				0x0001
#define PS3AV_CMD_AV_MCLK_512				0x0003
/* av_inputlen */
#define PS3AV_CMD_AV_INPUTLEN_16			0x02
#define PS3AV_CMD_AV_INPUTLEN_20			0x0a
#define PS3AV_CMD_AV_INPUTLEN_24			0x0b
/* alayout */
#define PS3AV_CMD_AV_LAYOUT_32				(1 << 0)
#define PS3AV_CMD_AV_LAYOUT_44				(1 << 1)
#define PS3AV_CMD_AV_LAYOUT_48				(1 << 2)
#define PS3AV_CMD_AV_LAYOUT_88				(1 << 3)
#define PS3AV_CMD_AV_LAYOUT_96				(1 << 4)
#define PS3AV_CMD_AV_LAYOUT_176				(1 << 5)
#define PS3AV_CMD_AV_LAYOUT_192				(1 << 6)
/* hdmi_mode */
#define PS3AV_CMD_AV_HDMI_MODE_NORMAL			0xff
#define PS3AV_CMD_AV_HDMI_HDCP_OFF			0x01
#define PS3AV_CMD_AV_HDMI_EDID_PASS			0x80
#define PS3AV_CMD_AV_HDMI_DVI				0x40


/* for video module */
/* video_head */
#define PS3AV_CMD_VIDEO_HEAD_A				0x0000
#define PS3AV_CMD_VIDEO_HEAD_B				0x0001
/* video_cs_out video_cs_in */
#define PS3AV_CMD_VIDEO_CS_NONE				0x0000
#define PS3AV_CMD_VIDEO_CS_RGB_8			0x0001
#define PS3AV_CMD_VIDEO_CS_YUV444_8			0x0002
#define PS3AV_CMD_VIDEO_CS_YUV422_8			0x0003
#define PS3AV_CMD_VIDEO_CS_XVYCC_8			0x0004
#define PS3AV_CMD_VIDEO_CS_RGB_10			0x0005
#define PS3AV_CMD_VIDEO_CS_YUV444_10			0x0006
#define PS3AV_CMD_VIDEO_CS_YUV422_10			0x0007
#define PS3AV_CMD_VIDEO_CS_XVYCC_10			0x0008
#define PS3AV_CMD_VIDEO_CS_RGB_12			0x0009
#define PS3AV_CMD_VIDEO_CS_YUV444_12			0x000a
#define PS3AV_CMD_VIDEO_CS_YUV422_12			0x000b
#define PS3AV_CMD_VIDEO_CS_XVYCC_12			0x000c
/* video_vid */
#define PS3AV_CMD_VIDEO_VID_NONE			0x0000
#define PS3AV_CMD_VIDEO_VID_480I			0x0001
#define PS3AV_CMD_VIDEO_VID_576I			0x0003
#define PS3AV_CMD_VIDEO_VID_480P			0x0005
#define PS3AV_CMD_VIDEO_VID_576P			0x0006
#define PS3AV_CMD_VIDEO_VID_1080I_60HZ			0x0007
#define PS3AV_CMD_VIDEO_VID_1080I_50HZ			0x0008
#define PS3AV_CMD_VIDEO_VID_720P_60HZ			0x0009
#define PS3AV_CMD_VIDEO_VID_720P_50HZ			0x000a
#define PS3AV_CMD_VIDEO_VID_1080P_60HZ			0x000b
#define PS3AV_CMD_VIDEO_VID_1080P_50HZ			0x000c
#define PS3AV_CMD_VIDEO_VID_WXGA			0x000d
#define PS3AV_CMD_VIDEO_VID_SXGA			0x000e
#define PS3AV_CMD_VIDEO_VID_WUXGA			0x000f
#define PS3AV_CMD_VIDEO_VID_480I_A			0x0010
/* video_format */
#define PS3AV_CMD_VIDEO_FORMAT_BLACK			0x0000
#define PS3AV_CMD_VIDEO_FORMAT_ARGB_8BIT		0x0007
/* video_order */
#define PS3AV_CMD_VIDEO_ORDER_RGB			0x0000
#define PS3AV_CMD_VIDEO_ORDER_BGR			0x0001
/* video_fmt */
#define PS3AV_CMD_VIDEO_FMT_X8R8G8B8			0x0000
/* video_out_format */
#define PS3AV_CMD_VIDEO_OUT_FORMAT_RGB_12BIT		0x0000
/* video_sync */
#define PS3AV_CMD_VIDEO_SYNC_VSYNC			0x0001
#define PS3AV_CMD_VIDEO_SYNC_CSYNC			0x0004
#define PS3AV_CMD_VIDEO_SYNC_HSYNC			0x0010

/* for audio module */
/* num_of_ch */
#define PS3AV_CMD_AUDIO_NUM_OF_CH_2			0x0000
#define PS3AV_CMD_AUDIO_NUM_OF_CH_3			0x0001
#define PS3AV_CMD_AUDIO_NUM_OF_CH_4			0x0002
#define PS3AV_CMD_AUDIO_NUM_OF_CH_5			0x0003
#define PS3AV_CMD_AUDIO_NUM_OF_CH_6			0x0004
#define PS3AV_CMD_AUDIO_NUM_OF_CH_7			0x0005
#define PS3AV_CMD_AUDIO_NUM_OF_CH_8			0x0006
/* audio_fs */
#define PS3AV_CMD_AUDIO_FS_32K				0x0001
#define PS3AV_CMD_AUDIO_FS_44K				0x0002
#define PS3AV_CMD_AUDIO_FS_48K				0x0003
#define PS3AV_CMD_AUDIO_FS_88K				0x0004
#define PS3AV_CMD_AUDIO_FS_96K				0x0005
#define PS3AV_CMD_AUDIO_FS_176K				0x0006
#define PS3AV_CMD_AUDIO_FS_192K				0x0007
/* audio_word_bits */
#define PS3AV_CMD_AUDIO_WORD_BITS_16			0x0001
#define PS3AV_CMD_AUDIO_WORD_BITS_20			0x0002
#define PS3AV_CMD_AUDIO_WORD_BITS_24			0x0003
/* audio_format */
#define PS3AV_CMD_AUDIO_FORMAT_PCM			0x0001
#define PS3AV_CMD_AUDIO_FORMAT_BITSTREAM		0x00ff
/* audio_source */
#define PS3AV_CMD_AUDIO_SOURCE_SERIAL			0x0000
#define PS3AV_CMD_AUDIO_SOURCE_SPDIF			0x0001
/* audio_swap */
#define PS3AV_CMD_AUDIO_SWAP_0				0x0000
#define PS3AV_CMD_AUDIO_SWAP_1				0x0000
/* audio_map */
#define PS3AV_CMD_AUDIO_MAP_OUTPUT_0			0x0000
#define PS3AV_CMD_AUDIO_MAP_OUTPUT_1			0x0001
#define PS3AV_CMD_AUDIO_MAP_OUTPUT_2			0x0002
#define PS3AV_CMD_AUDIO_MAP_OUTPUT_3			0x0003
/* audio_layout */
#define PS3AV_CMD_AUDIO_LAYOUT_2CH			0x0000
#define PS3AV_CMD_AUDIO_LAYOUT_6CH			0x000b /* LREClr */
#define PS3AV_CMD_AUDIO_LAYOUT_8CH			0x001f /* LREClrXY */
/* audio_downmix */
#define PS3AV_CMD_AUDIO_DOWNMIX_PERMITTED		0x0000
#define PS3AV_CMD_AUDIO_DOWNMIX_PROHIBITED		0x0001

/* audio_port */
#define PS3AV_CMD_AUDIO_PORT_HDMI_0			( 1 << 0 )
#define PS3AV_CMD_AUDIO_PORT_HDMI_1			( 1 << 1 )
#define PS3AV_CMD_AUDIO_PORT_AVMULTI_0			( 1 << 10 )
#define PS3AV_CMD_AUDIO_PORT_SPDIF_0			( 1 << 20 )
#define PS3AV_CMD_AUDIO_PORT_SPDIF_1			( 1 << 21 )

/* audio_ctrl_id */
#define PS3AV_CMD_AUDIO_CTRL_ID_DAC_RESET		0x0000
#define PS3AV_CMD_AUDIO_CTRL_ID_DAC_DE_EMPHASIS		0x0001
#define PS3AV_CMD_AUDIO_CTRL_ID_AVCLK			0x0002
/* audio_ctrl_data[0] reset */
#define PS3AV_CMD_AUDIO_CTRL_RESET_NEGATE		0x0000
#define PS3AV_CMD_AUDIO_CTRL_RESET_ASSERT		0x0001
/* audio_ctrl_data[0] de-emphasis */
#define PS3AV_CMD_AUDIO_CTRL_DE_EMPHASIS_OFF		0x0000
#define PS3AV_CMD_AUDIO_CTRL_DE_EMPHASIS_ON		0x0001
/* audio_ctrl_data[0] avclk */
#define PS3AV_CMD_AUDIO_CTRL_AVCLK_22			0x0000
#define PS3AV_CMD_AUDIO_CTRL_AVCLK_18			0x0001

/* av_vid */
/* do not use these params directly, use vid_video2av */
#define PS3AV_CMD_AV_VID_480I				0x0000
#define PS3AV_CMD_AV_VID_480P				0x0001
#define PS3AV_CMD_AV_VID_720P_60HZ			0x0002
#define PS3AV_CMD_AV_VID_1080I_60HZ			0x0003
#define PS3AV_CMD_AV_VID_1080P_60HZ			0x0004
#define PS3AV_CMD_AV_VID_576I				0x0005
#define PS3AV_CMD_AV_VID_576P				0x0006
#define PS3AV_CMD_AV_VID_720P_50HZ			0x0007
#define PS3AV_CMD_AV_VID_1080I_50HZ			0x0008
#define PS3AV_CMD_AV_VID_1080P_50HZ			0x0009
#define PS3AV_CMD_AV_VID_WXGA				0x000a
#define PS3AV_CMD_AV_VID_SXGA				0x000b
#define PS3AV_CMD_AV_VID_WUXGA				0x000c
/* av_cs_out av_cs_in */
/* use cs_video2av() */
#define PS3AV_CMD_AV_CS_RGB_8				0x0000
#define PS3AV_CMD_AV_CS_YUV444_8			0x0001
#define PS3AV_CMD_AV_CS_YUV422_8			0x0002
#define PS3AV_CMD_AV_CS_XVYCC_8				0x0003
#define PS3AV_CMD_AV_CS_RGB_10				0x0004
#define PS3AV_CMD_AV_CS_YUV444_10			0x0005
#define PS3AV_CMD_AV_CS_YUV422_10			0x0006
#define PS3AV_CMD_AV_CS_XVYCC_10			0x0007
#define PS3AV_CMD_AV_CS_RGB_12				0x0008
#define PS3AV_CMD_AV_CS_YUV444_12			0x0009
#define PS3AV_CMD_AV_CS_YUV422_12			0x000a
#define PS3AV_CMD_AV_CS_XVYCC_12			0x000b
#define PS3AV_CMD_AV_CS_8				0x0000
#define PS3AV_CMD_AV_CS_10				0x0001
#define PS3AV_CMD_AV_CS_12				0x0002
/* dither */
#define PS3AV_CMD_AV_DITHER_OFF				0x0000
#define PS3AV_CMD_AV_DITHER_ON				0x0001
#define PS3AV_CMD_AV_DITHER_8BIT			0x0000
#define PS3AV_CMD_AV_DITHER_10BIT			0x0002
#define PS3AV_CMD_AV_DITHER_12BIT			0x0004
/* super_white */
#define PS3AV_CMD_AV_SUPER_WHITE_OFF			0x0000
#define PS3AV_CMD_AV_SUPER_WHITE_ON			0x0001
/* aspect */
#define PS3AV_CMD_AV_ASPECT_16_9			0x0000
#define PS3AV_CMD_AV_ASPECT_4_3				0x0001
/* video_cs_cnv() */
#define PS3AV_CMD_VIDEO_CS_RGB				0x0001
#define PS3AV_CMD_VIDEO_CS_YUV422			0x0002
#define PS3AV_CMD_VIDEO_CS_YUV444			0x0003

/* for automode */
#define PS3AV_RESBIT_720x480P			0x0003 /* 0x0001 | 0x0002 */
#define PS3AV_RESBIT_720x576P			0x0003 /* 0x0001 | 0x0002 */
#define PS3AV_RESBIT_1280x720P			0x0004
#define PS3AV_RESBIT_1920x1080I			0x0008
#define PS3AV_RESBIT_1920x1080P			0x4000
#define PS3AV_RES_MASK_60			(PS3AV_RESBIT_720x480P \
						| PS3AV_RESBIT_1280x720P \
						| PS3AV_RESBIT_1920x1080I \
						| PS3AV_RESBIT_1920x1080P) 
#define PS3AV_RES_MASK_50			(PS3AV_RESBIT_720x576P \
						| PS3AV_RESBIT_1280x720P \
						| PS3AV_RESBIT_1920x1080I \
						| PS3AV_RESBIT_1920x1080P)

#define PS3AV_MONITOR_TYPE_HDMI			1 /* HDMI */
#define PS3AV_MONITOR_TYPE_DVI			2 /* DVI */
#define PS3AV_DEFAULT_HDMI_VID_REG_60		PS3AV_CMD_VIDEO_VID_480P
#define PS3AV_DEFAULT_AVMULTI_VID_REG_60	PS3AV_CMD_VIDEO_VID_480I
#define PS3AV_DEFAULT_HDMI_VID_REG_50		PS3AV_CMD_VIDEO_VID_576P
#define PS3AV_DEFAULT_AVMULTI_VID_REG_50	PS3AV_CMD_VIDEO_VID_576I
#define PS3AV_DEFAULT_DVI_VID			PS3AV_CMD_VIDEO_VID_480P

#define PS3AV_BOOT_NTSC				0x00
#define PS3AV_BOOT_PAL_RGB			0x01
#define PS3AV_BOOT_PAL_YUV			0x02

#define PS3AV_REGION_60				0x01
#define PS3AV_REGION_50				0x02
#define PS3AV_REGION_RGB			0x10

#define get_status(buf)				(((uint32_t *)buf)[2])
#define PS3AV_HDR_SIZE				4 /* version + size */

/* for video mode */
#define PS3AV_MODE_MASK				0x000F
#define PS3AV_MODE_HDCP_OFF			0x1000 /* PS3 tool only */
#define PS3AV_MODE_DITHER			0x0800
#define PS3AV_MODE_FULL				0x0080
#define PS3AV_MODE_DVI				0x0040
#define PS3AV_MODE_RGB				0x0020

#ifdef __KERNEL__
/** command packet structure **/
/* backend: initialization */
struct ps3av_pkt_av_init {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint32_t event_bit; 
} __attribute__ ((packed));

/* backend: finalize */
struct ps3av_pkt_av_fin {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	/* recv */
	uint32_t reserved; 
} __attribute__ ((packed));

/* backend: get port */
struct ps3av_pkt_av_get_hw_conf {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	/* recv */
	uint32_t status;
	uint16_t num_of_hdmi;		/* out: number of hdmi */
	uint16_t num_of_avmulti;	/* out: number of avmulti */
	uint16_t num_of_spdif;		/* out: number of hdmi */
	uint16_t reserved;
} __attribute__ ((packed));

/* backend: get monitor info */
struct ps3av_pkt_av_get_monitor_info {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint16_t avport;		/* in: avport */
	uint16_t reserved;
	/* recv */
	uint8_t  info[256];		/* out: monitor info */
} __attribute__ ((packed));

/* backend: enable/disable event */
struct ps3av_pkt_av_event {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint32_t event_bit;		/* in */
} __attribute__ ((packed));

/* backend: video cs param */
struct ps3av_pkt_av_video_cs {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint16_t avport;		/* in: avport */
	uint16_t av_vid;		/* in: video resolution */
	uint16_t av_cs_out;		/* in: output color space */
	uint16_t av_cs_in;		/* in: input color space */
	uint8_t  dither;		/* in: dither bit length */
	uint8_t  bitlen_out;		/* in: bit length */
	uint8_t  super_white;		/* in: super white */
	uint8_t  aspect;		/* in: aspect ratio */
} __attribute__ ((packed));

/* backend: video mute */
struct ps3av_av_mute {
	uint16_t avport;		/* in: avport */
	uint16_t mute;			/* in: mute on/off */
} __attribute__ ((packed));
struct ps3av_pkt_av_video_mute {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	struct ps3av_av_mute mute[PS3AV_MUTE_PORT_MAX];
} __attribute__ ((packed));

/* backend: video disable signal */
struct ps3av_pkt_av_video_disable_sig {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint16_t avport;		/* in: avport */
	uint16_t reserved;
} __attribute__ ((packed));

/* backend: audio param */
struct ps3av_pkt_av_audio_param {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint16_t avport;		/* in: avport */
	uint16_t reserved;
	uint8_t  mclk;			/* in: audio mclk */
	uint8_t  ns[3];			/* in: audio ns val */
	uint8_t  enable;		/* in: audio enable */
	uint8_t  swaplr;		/* in: audio swap */
	uint8_t  fifomap;		/* in: audio fifomap */
	uint8_t  inputctrl;		/* in: audio input ctrl */
	uint8_t  inputlen;		/* in: sample bit size */
	uint8_t  layout;		/* in: speaker layout param*/
	uint8_t  info[5];		/* in: info */
	uint8_t  chstat[5];		/* in: ch stat */
} __attribute__ ((packed));
struct ps3av_audio_info_frame {
	struct pb1_bit {
		uint8_t ct:4;
		uint8_t rsv:1;
		uint8_t cc:3;
	} pb1;
	struct pb2_bit {
		uint8_t rsv:3;
		uint8_t sf:8;
		uint8_t ss:2;
	} pb2;
	uint8_t pb3;
	uint8_t pb4;
	struct pb5_bit {
		uint8_t dm:1;
		uint8_t lsv:4;
		uint8_t rsv:3;
	} pb5;
} __attribute__ ((packed));
/* backend: audio_mute */
struct ps3av_pkt_av_audio_mute {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	struct ps3av_av_mute mute[PS3AV_MUTE_PORT_MAX];
} __attribute__ ((packed));

/* backend: hdmi_mode */
struct ps3av_pkt_av_hdmi_mode {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint8_t  mode;			/* in: hdmi_mode */
	uint8_t  reserved0;
	uint8_t  reserved1;
	uint8_t  reserved2;
} __attribute__ ((packed));

/* backend: tv_mute */
struct ps3av_pkt_av_tv_mute {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint16_t avport;		/* in: avport HDMI only */
	uint16_t mute;			/* in: mute */
} __attribute__ ((packed));


/* video: initialize */
struct ps3av_pkt_video_init {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	/* recv */
	uint32_t reserved; 
} __attribute__ ((packed));

/* video: mode setting */
struct ps3av_pkt_video_mode {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint32_t video_head;		/* in: head */
	uint32_t reserved;
	uint32_t video_vid;		/* in: video resolution */
	uint16_t reserved1;
	uint16_t width;			/* in: width in pixel */
	uint16_t reserved2;
	uint16_t height;		/* in: height in pixel */
	uint32_t pitch;			/* in: line size in byte*/
	uint32_t video_out_format;	/* in: out format */
	uint32_t video_format;		/* in: input frame buffer format */
	uint8_t  reserved3;
	uint8_t  reserved4;
	uint16_t video_order;		/* in: input RGB order */
	uint32_t reserved5;
} __attribute__ ((packed));

/* video: format */
struct ps3av_pkt_video_format {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint32_t video_head;		/* in: head */
	uint32_t video_format;		/* in: frame buffer format */
	uint16_t reserved;
	uint16_t video_order;		/* in: input RGB order */
} __attribute__ ((packed));
/* video: pitch */
struct ps3av_pkt_video_pitch {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint32_t video_head;		/* in: head */
	uint32_t pitch;			/* in: line size in byte */
} __attribute__ ((packed));


/* audio: initialize */
struct ps3av_pkt_audio_init {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	/* recv */
	uint32_t reserved; 
} __attribute__ ((packed));

/* audio: mode setting */
struct ps3av_pkt_audio_mode {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint8_t  avport;		/* in: avport */
	uint8_t  reserved0[3];
	uint32_t mask;			/* in: mask */
	uint32_t audio_num_of_ch;	/* in: number of ch */
	uint32_t audio_fs;		/* in: sampling freq */
	uint32_t audio_word_bits;	/* in: sample bit size */
	uint32_t audio_format;		/* in: audio output format */
	uint32_t audio_source;		/* in: audio source */
	uint8_t  audio_enable[4];	/* in: audio enable */
	uint8_t  audio_swap[4];		/* in: audio swap */
	uint8_t  audio_map[4];		/* in: audio map */
	uint32_t audio_layout;		/* in: speaker layout */
	uint32_t audio_downmix;		/* in: audio downmix permission */
	uint32_t audio_downmix_level;
	uint8_t  audio_cs_info[8];	/* in: IEC channel status */
} __attribute__ ((packed));

/* audio: mute */
struct ps3av_audio_mute {
	uint8_t  avport;		/* in: opt_port optical */
	uint8_t  reserved[3];
	uint32_t mute;			/* in: mute */
} __attribute__ ((packed));
struct ps3av_pkt_audio_mute {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;				/* command id */
	struct ps3av_audio_mute mute[PS3AV_OPT_PORT_MAX];
} __attribute__ ((packed));

/* audio: active/inactive */
struct ps3av_pkt_audio_active {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint32_t audio_port;		/* in: audio active/inactive port */
} __attribute__ ((packed));

/* audio: SPDIF user bit */
struct ps3av_pkt_audio_spdif_bit {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint8_t  avport;		/* in: avport SPDIF only */
	uint8_t  reserved[3];
	uint32_t audio_port;		/* in: SPDIF only */
	uint32_t spdif_bit_data[12];	/* in: user bit data */
} __attribute__ ((packed));

/* audio: audio control */
struct ps3av_pkt_audio_ctrl {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint32_t audio_ctrl_id;		/* in: control id */
	uint32_t audio_ctrl_data[4];	/* in: control data */
} __attribute__ ((packed));


/* monitor information */
struct ps3av_info_resolution {
	uint32_t res_bits;
	uint32_t native;
} __attribute__ ((packed));
struct ps3av_info_cs {
	uint8_t rgb;
	uint8_t yuv444;
	uint8_t yuv422;
	uint8_t reserved;
} __attribute__ ((packed));
struct ps3av_info_color {
	uint16_t red_x;
	uint16_t red_y;
	uint16_t green_x;
	uint16_t green_y;
	uint16_t blue_x;
	uint16_t blue_y;
	uint16_t white_x;
	uint16_t white_y;
	uint32_t gamma;
} __attribute__ ((packed));
struct ps3av_info_audio {
	uint8_t type;
	uint8_t max_num_of_ch;
	uint8_t fs;
	uint8_t sbit;
} __attribute__ ((packed));
struct ps3av_info_monitor {
	uint8_t avport;
	uint8_t monitor_id[10];
	uint8_t monitor_type;
	uint8_t monitor_name[16];
	struct ps3av_info_resolution res_60;
	struct ps3av_info_resolution res_50;
	struct ps3av_info_resolution res_other;
	struct ps3av_info_resolution res_vesa;
	struct ps3av_info_cs cs;
	struct ps3av_info_color color;
	uint8_t supported_ai;
	uint8_t speaker_info;
	uint8_t num_of_audio_block;
} __attribute__ ((packed));


/* avb:param */
struct ps3av_pkt_avb_param {
	uint16_t version;
	uint16_t size;			/* size of command packet */
	uint32_t cid;			/* command id */
	uint16_t num_of_video_pkt;
	uint16_t num_of_audio_pkt;
	uint16_t num_of_av_video_pkt;
	uint16_t num_of_av_audio_pkt;
	struct ps3av_pkt_video_mode video[PS3AV_AVB_NUM_VIDEO];
	struct ps3av_pkt_audio_mode audio[PS3AV_AVB_NUM_AUDIO];
	struct ps3av_pkt_av_video_cs av_video[PS3AV_AVB_NUM_AV_VIDEO];
	struct ps3av_pkt_av_audio_param av_audio[PS3AV_AVB_NUM_AV_AUDIO];
} __attribute__ ((packed));


/* vuart */
struct vuart {
	unsigned long port;
	unsigned long bufsize;
	unsigned long page_shift;
	unsigned long lpar_addr;
	void *buf;
};


struct ps3av {
	int available;
	struct semaphore sem;
	struct vuart vuart;

	int region;
	struct ps3av_pkt_av_get_hw_conf av_hw_conf;
	uint32_t av_port[PS3AV_AV_PORT_MAX + PS3AV_OPT_PORT_MAX];
	uint32_t opt_port[PS3AV_OPT_PORT_MAX];
	uint32_t head[PS3AV_HEAD_MAX];
	uint32_t audio_port;
	atomic_t ps3av_mode;
};

/** command status **/
#define PS3AV_STATUS_SUCCESS			0x0000 /* success */
#define PS3AV_STATUS_RECEIVE_VUART_ERROR	0x0001 /* receive vuart error */
#define PS3AV_STATUS_SYSCON_COMMUNICATE_FAIL	0x0002 /* syscon communication error */
#define PS3AV_STATUS_INVALID_COMMAND		0x0003 /* obsolete invalid CID */
#define PS3AV_STATUS_INVALID_PORT		0x0004 /* invalid port number */
#define PS3AV_STATUS_INVALID_VID		0x0005 /* invalid video format */
#define PS3AV_STATUS_INVALID_COLOR_SPACE	0x0006 /* invalid video colose space */
#define PS3AV_STATUS_INVALID_FS			0x0007 /* invalid audio sampling freq */
#define PS3AV_STATUS_INVALID_AUDIO_CH		0x0008 /* invalid audio channel number */
#define PS3AV_STATUS_UNSUPPORTED_VERSION	0x0009 /* version mismatch  */
#define PS3AV_STATUS_INVALID_SAMPLE_SIZE	0x000a /* invalid audio sample bit size */
#define PS3AV_STATUS_FAILURE			0x000b /* other failures */
#define PS3AV_STATUS_UNSUPPORTED_COMMAND	0x000c /* unsupported cid */
#define PS3AV_STATUS_BUFFER_OVERFLOW		0x000d /* write buffer overflow */
#define PS3AV_STATUS_INVALID_VIDEO_PARAM	0x000e /* invalid video param */
#define PS3AV_STATUS_NO_SEL			0x000f /* not exist selector */
#define PS3AV_STATUS_INVALID_AV_PARAM		0x0010 /* invalid backend param */
#define PS3AV_STATUS_INVALID_AUDIO_PARAM	0x0011 /* invalid audio param */
#define PS3AV_STATUS_UNSUPPORTED_HDMI_MODE	0x0012 /* unsupported hdmi mode */
#define PS3AV_STATUS_NO_SYNC_HEAD		0x0013 /* sync head failed */

extern void ps3av_set_hdr(uint32_t, uint16_t, void *);
extern int ps3av_do_pkt(uint32_t, uint16_t, size_t, uint32_t *);

extern int ps3av_cmd_init(void);
extern int ps3av_cmd_fin(void);
extern int ps3av_cmd_av_video_mute(int, uint32_t *, uint32_t);
extern int ps3av_cmd_av_video_disable_sig(uint32_t);
extern int ps3av_cmd_av_tv_mute(uint32_t, uint32_t);
extern int ps3av_cmd_enable_event(void);
extern int ps3av_cmd_av_hdmi_mode(uint8_t);
extern uint8_t *ps3av_cmd_set_av_video_cs(uint8_t *, uint32_t, int, int, int, uint32_t);
extern uint8_t *ps3av_cmd_set_video_mode(uint8_t *, uint32_t, int, int, uint32_t);
extern int ps3av_cmd_video_format_black(uint32_t, uint32_t, uint32_t);
extern int ps3av_cmd_av_audio_mute(int, uint32_t *, uint32_t);
extern uint8_t *ps3av_cmd_set_av_audio_param(uint8_t *, uint32_t, struct ps3av_pkt_audio_mode *, uint32_t);
extern uint8_t *ps3av_cmd_set_audio_mode(uint8_t *, uint32_t, uint32_t, uint32_t, uint32_t, uint32_t, uint32_t);
extern int ps3av_cmd_audio_mode(struct ps3av_pkt_audio_mode *);
extern int ps3av_cmd_audio_mute(int , uint32_t *, uint32_t);
extern int ps3av_cmd_audio_active(int, uint32_t);
extern int ps3av_cmd_avb_param(struct ps3av_pkt_avb_param *, uint32_t);
extern int ps3av_cmd_av_get_hw_conf(struct ps3av_pkt_av_get_hw_conf *);
extern int ps3av_cmd_av_hw_conf_dump(struct ps3av_pkt_av_get_hw_conf *);
extern int ps3av_cmd_video_get_monitor_info(struct ps3av_pkt_av_get_monitor_info *, uint32_t);
extern int ps3av_cmd_av_monitor_info_dump(struct ps3av_pkt_av_get_monitor_info *);

extern int ps3av_vuart_init(struct vuart *, unsigned long, int);
extern int ps3av_vuart_cleanup(struct vuart *);
extern int ps3av_vuart_write(struct vuart *, void *, unsigned long);
extern int ps3av_vuart_read(struct vuart *, void *, unsigned long, int);

extern int ps3av_set_video_mode(uint32_t, int);
extern int ps3av_set_audio_mode(uint32_t, uint32_t, uint32_t, uint32_t, uint32_t);
extern int ps3av_set_mode(uint32_t, int);
extern int ps3av_get_mode(void);
extern int ps3av_get_scanmode(int);
extern int ps3av_get_refresh_rate(int);
extern int ps3av_video_mode2res(uint32_t, uint32_t *, uint32_t *);
extern int ps3av_video_mute(int);
extern int ps3av_audio_mute(int);
extern int ps3av_dev_open(void);
extern int ps3av_dev_close(void);
#endif //__KERNEL__

#endif //  _PS3AV_H_
